set echo on
create table deposits (customer,amount,the_date)
as
select 1, 1000, date '2003-01-01' from dual union all
select 1, 200, date '2004-01-01' from dual union all
select 1, 500, date '2005-01-01' from dual union all
select 1, 100, date '2006-01-01' from dual union all
select 1, 800, date '2007-01-01' from dual union all
select 2, 20, date '2003-01-01' from dual union all
select 2, 150, date '2004-01-01' from dual union all
select 2, 60, date '2005-01-01' from dual union all
select 2, 100, date '2006-01-01' from dual union all
select 2, 100, date '2007-01-01' from dual
/
create table interest_rates (the_date,percentage)
as
select date '2003-01-01', 5 from dual union all
select date '2004-01-01', 3.2 from dual union all
select date '2005-01-01', 4.1 from dual union all
select date '2006-01-01', 5.8 from dual union all
select date '2007-01-01', 4.9 from dual
/
pause
select * from deposits
/
pause
select * from interest_rates
/
pause
select customer
     , the_date
     , balance balance_at_end_of_year
  from deposits s
     , interest_rates r
 where s.the_date = r.the_date
 model
       partition by (s.customer)
       dimension by (s.the_date)
       measures (s.amount, r.percentage, 0 balance)
       rules
       ( balance[any] order by the_date
         = round
           (   (nvl(balance[add_months(cv(),-12)],0) + amount[cv()])
             * (1 + percentage[cv()]/100)
           , 2
           )
       )
 order by customer
     , the_date
/
pause
clear screen
with t as
( select s.customer
       , r.the_date
       , 1 + r.percentage/100 factor
       , s.amount
    from deposits s
       , interest_rates r
   where s.the_date = r.the_date
)
, t_recursive (customer,the_date,amount) as
( select customer
       , min(the_date)
       , min(amount * factor) keep (dense_rank first order by the_date)
    from t
   group by customer
   union all
  select t.customer
       , t.the_date
       , round((tr.amount + t.amount) * t.factor,2)
    from t
       , t_recursive tr
   where tr.customer = t.customer
     and tr.the_date = t.the_date - interval '1' year
)
select customer
     , the_date
     , amount balance_at_end_of_year
  from t_recursive
 order by customer
     , the_date
/
pause
set echo off
drop table interest_rates purge
/
drop table deposits purge
/
